# Generated by Django 2.2.3 on 2019-07-27 16:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0031_auto_20190727_1639'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statementrecorddetail',
            name='unstructured_remittance_info',
            field=models.CharField(blank=True, max_length=2048, verbose_name='unstructured remittance info'),
        ),
    ]



from datetime import datetime, date
from decimal import Decimal
from pprint import pprint
from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import transaction
from django.db.models import QuerySet
from django.utils.dateparse import parse_date
from django.utils.translation import ugettext as _
from jacc.models import Account, EntryType
from jutil.format import dec2, dec4
from jutil.parse import parse_datetime

from jbank.models import StatementFile, Statement, StatementRecord, DELIVERY_FROM_BANK_SYSTEM, \
    StatementRecordDetail, CurrencyExchange, StatementRecordRemittanceInfo
from jbank.parsers import parse_filename_suffix


CAMT053_STATEMENT_SUFFIXES = ('XML', 'XT', 'CAMT')

CAMT053_ARRAY_TAGS = ['Bal', 'Ntry', 'NtryDtls', 'TxDtls', 'Strd']

CAMT053_INT_TAGS = ['NbOfNtries', 'NbOfTxs']


def camt053_get_iban(data: dict) -> str:
    return data.get('BkToCstmrStmt', {}).get('Stmt', {}).get('Acct', {}).get('Id', {}).get('IBAN', '')


def camt053_get_val(data: dict, key: str, default='', required: bool = True, name: str = ''):
    if key not in data:
        if required:
            raise ValidationError(_('camt.053 field {} missing').format(name if name else key))
        return default
    return data[key]


def camt053_get_currency(data: dict, key: str, required: bool = True, name: str = ''):
    try:
        v = camt053_get_val(data, key, default=None, required=False, name=name)
        if v:
            amount = dec2(v['@'])
            currency_code = v['@Ccy']
            return amount, currency_code
    except:
        pass
    if required:
        raise ValidationError(_('camt.053 field {} type {} missing or invalid').format(name, 'currency'))
    return None, ''


def camt053_get_dt(data: dict, key: str, default: datetime or None = None, required: bool = True, name: str = '') -> datetime or None:
    s = camt053_get_val(data, key, None, required, name)
    return parse_datetime(s) if s is not None else default


def camt053_get_int(data: dict, key: str, default: int or None = None, required: bool = True, name: str = '') -> int:
    s = camt053_get_val(data, key, default, required, name)
    try:
        return int(s)
    except:
        pass
    raise ValidationError(_('camt.053 field {} type {} missing or invalid').format(name, 'int'))


def camt053_get_date(data: dict, key: str, default: date or None = None, required: bool = True, name: str = '') -> date:
    s = camt053_get_val(data, key, default, required, name)
    try:
        return parse_date(s)
    except:
        pass
    raise ValidationError(_('camt.053 field {} type {} missing or invalid').format(name, 'date'))


def camt053_parse_statement_from_file(filename: str) -> dict:
    from jutil.xml import xml_to_dict

    if parse_filename_suffix(filename).upper() not in CAMT053_STATEMENT_SUFFIXES:
        raise ValidationError(_('File {} has unrecognized ({}) suffix for file type "{}"').format(filename, ', '.join(CAMT053_STATEMENT_SUFFIXES, 'camt.053')))
    with open(filename, 'rb') as fp:
        data = xml_to_dict(fp.read(), array_tags=CAMT053_ARRAY_TAGS, int_tags=CAMT053_INT_TAGS)
        return data


def camt053_get_stmt_bal(d_stmt: dict, bal_type: str) -> (Decimal, date):
    for bal in d_stmt.get('Bal', []):
        if bal.get('Tp', {}).get('CdOrPrtry', {}).get('Cd', '') == bal_type:
            amt = Decimal(bal.get('Amt', {}).get('@', ''))
            dt = camt053_get_date(bal.get('Dt', {}), 'Dt', name='Stmt.Bal[{}].Dt.Dt'.format(bal_type))
            return amt, dt
    raise ValidationError(_('camt.053 field {} type {} missing or invalid').format('Stmt.Bal.Tp.CdOrPrty.Cd', bal_type))


def camt053_domain_from_record_code(record_domain: str) -> str:
    if record_domain == 'PMNT':
        return '700'
    elif record_domain == 'LDAS':
        return '761'
    return ''


def camt053_get_unified_val(qs, k: str, default) -> str:
    v = default
    for e in qs:
        v2 = getattr(e, k)
        if v == default:
            v = v2
        elif v and v2 and v2 != v:
            return default
    return v


def camt053_get_unified_str(qs, k: str) -> str:
    return camt053_get_unified_val(qs, k, '')


@transaction.atomic
def camt053_create_statement(statement_data: dict, name: str, file: StatementFile, **kw) -> Statement:
    """
    Creates camt.053 Statement from statement data parsed by camt053_parse_statement_from_file()
    :param statement_data: XML data in form of dict
    :param name: File name of the account statement
    :param file: Source statement file
    :return: Statement
    """
    account_number = camt053_get_iban(statement_data)
    if not account_number:
        raise ValidationError('{name}: '.format(name=name) + _("account.not.found").format(account_number=''))
    accounts = list(Account.objects.filter(name=account_number))
    if len(accounts) != 1:
        raise ValidationError('{name}: '.format(name=name) + _("account.not.found").format(account_number=account_number))
    account = accounts[0]
    assert isinstance(account, Account)

    d_stmt = statement_data.get('BkToCstmrStmt', {}).get('Stmt', {})
    d_acct = d_stmt.get('Acct', {})
    d_ownr = d_acct.get('Ownr', {})
    d_ntry = d_stmt.get('Ntry', [])
    d_frto = d_stmt.get('FrToDt', {})
    d_txsummary = d_stmt.get('TxsSummry', {})

    if Statement.objects.filter(name=name, account=account).first():
        raise ValidationError('Bank account {} statement {} of processed already'.format(account_number, name))
    stm = Statement(name=name, account=account, file=file)
    stm.account_number = stm.iban = account_number
    stm.bic = camt053_get_val(d_acct.get('Svcr', {}).get('FinInstnId', {}), 'BIC', name='Stmt.Acct.Svcr.FinInstnId.BIC')
    stm.statement_identifier = camt053_get_val(d_stmt, 'Id', name='Stmt.Id')
    stm.statement_number = camt053_get_val(d_stmt, 'LglSeqNb', name='Stmt.LglSeqNb')
    stm.record_date = camt053_get_dt(d_stmt, 'CreDtTm', name='Stmt.CreDtTm')
    stm.begin_date = camt053_get_dt(d_frto, 'FrDtTm', name='Stmt.FrDtTm').date()
    stm.end_date = camt053_get_dt(d_frto, 'ToDtTm', name='Stmt.ToDtTm').date()
    stm.currency_code = camt053_get_val(d_acct, 'Ccy', name='Stmt.Acct.Ccy')
    if stm.currency_code != account.currency:
        raise ValidationError(_('Account currency {account_currency} does not match statement entry currency {statement_currency}'.format(statement_currency=stm.currency_code, account_currency=account.currency)))
    stm.owner_name = camt053_get_val(d_ownr, 'Nm', name='Stm.Acct.Ownr.Nm')
    stm.begin_balance, stm.begin_balance_date = camt053_get_stmt_bal(d_stmt, 'OPBD')
    stm.record_count = camt053_get_int(d_txsummary.get('TtlNtries', {}), 'NbOfNtries', name='Stmt.TxsSummry.TtlNtries.NbOfNtries')
    stm.bank_specific_info_1 = camt053_get_val(d_stmt, 'AddtlStmtInf', required=False)[:1024]
    for k, v in kw.items():
        setattr(stm, k, v)
    stm.full_clean()
    stm.save()

    e_deposit = EntryType.objects.filter(code=settings.E_BANK_DEPOSIT).first()
    if not e_deposit:
        raise ValidationError(_('entry.type.missing') + ' ({}): {}'.format('settings.E_BANK_DEPOSIT', settings.E_BANK_DEPOSIT))
    assert isinstance(e_deposit, EntryType)
    e_withdraw = EntryType.objects.filter(code=settings.E_BANK_WITHDRAW).first()
    if not e_withdraw:
        raise ValidationError(_('entry.type.missing') + ' ({}): {}'.format('settings.E_BANK_WITHDRAW', settings.E_BANK_WITHDRAW))
    assert isinstance(e_withdraw, EntryType)
    e_types = {
        'CRDT': e_deposit,
        'DBIT': e_withdraw,
    }
    record_type_map = {
        'CRDT': '1',
        'DBIT': '2',
    }

    for ntry in d_ntry:
        archive_id = ntry.get('AcctSvcrRef', '')
        amount, cur = camt053_get_currency(ntry, 'Amt', name='Stmt.Ntry[{}].Amt'.format(archive_id))
        if cur != account.currency:
            raise ValidationError(_('Account currency {account_currency} does not match statement entry currency {statement_currency}'.format(statement_currency=cur, account_currency=account.currency)))

        cdt_dbt_ind = ntry['CdtDbtInd']
        e_type = e_types.get(cdt_dbt_ind, None)
        if not e_type:
            raise ValidationError(_('Statement entry type {} not supported').format(cdt_dbt_ind))

        rec = StatementRecord(statement=stm, account=account, type=e_type)
        rec.amount = amount
        rec.archive_identifier = archive_id
        rec.entry_type = record_type_map[cdt_dbt_ind]
        rec.record_date = record_date = camt053_get_date(ntry.get('BookgDt', {}), 'Dt', name='Stmt.Ntry[{}].BkkgDt.Dt'.format(archive_id))
        rec.value_date = camt053_get_date(ntry.get('ValDt', {}), 'Dt', name='Stmt.Ntry[{}].ValDt.Dt'.format(archive_id))
        rec.delivery_method = DELIVERY_FROM_BANK_SYSTEM

        d_bktxcd = ntry.get('BkTxCd', {})
        d_domn = d_bktxcd.get('Domn', {})
        d_family = d_domn.get('Fmly', {})
        d_prtry = d_bktxcd.get('Prtry', {})
        rec.record_domain = record_domain = camt053_get_val(d_domn, 'Cd', name='Stmt.Ntry[{}].BkTxCd.Domn.Cd'.format(archive_id))
        rec.record_code = camt053_domain_from_record_code(record_domain)
        rec.family_code = camt053_get_val(d_family, 'Cd', name='Stmt.Ntry[{}].BkTxCd.Domn.Family.Cd'.format(archive_id))
        rec.sub_family_code = camt053_get_val(d_family, 'SubFmlyCd', name='Stmt.Ntry[{}].BkTxCd.Domn.Family.SubFmlyCd'.format(archive_id))
        rec.record_description = camt053_get_val(d_prtry, 'Cd', required=False)

        rec.full_clean()
        rec.save()

        for dtl_batch in ntry.get('NtryDtls', []):
            batch_identifier = dtl_batch.get('Btch', {}).get('MsgId', '')
            dtl_ix = 0
            for dtl in dtl_batch.get('TxDtls', []):
                d = StatementRecordDetail(record=rec, batch_identifier=batch_identifier)

                d_amt_dtl = dtl.get('AmtDtls', {})
                d_txamt = d_amt_dtl.get('TxAmt', {})
                d_xchg = d_txamt.get('CcyXchg', None)

                d.amount, d.currency_code = camt053_get_currency(d_txamt, 'Amt', required=False)
                d.instructed_amount, source_currency = camt053_get_currency(d_amt_dtl.get('InstrdAmt', {}), 'Amt', required=False)
                if (not d_xchg and source_currency and source_currency != d.currency_code) or (d_xchg and not source_currency):
                    raise ValidationError(_('Inconsistent Stmt.Ntry[{}].NtryDtls.TxDtls[{}].AmtDtls'.format(archive_id, dtl_ix)))
                if source_currency and source_currency != d.currency_code:
                    source_currency = camt053_get_val(d_xchg, 'SrcCcy', default=source_currency, required=False)
                    target_currency = camt053_get_val(d_xchg, 'TrgCcy', default=d.currency_code, required=False)
                    unit_currency = camt053_get_val(d_xchg, 'UnitCcy', default='', required=False)
                    exchange_rate_str = camt053_get_val(d_xchg, 'XchgRate', default='', required=False)
                    exchange_rate = dec4(exchange_rate_str) if exchange_rate_str else None
                    d.exchange, created = CurrencyExchange.objects.get_or_create(record_date=record_date, source_currency=source_currency, target_currency=target_currency, unit_currency=unit_currency, exchange_rate=exchange_rate)

                d_refs = dtl.get('Refs', {})
                d.archive_identifier = d_refs.get('AcctSvcrRef', '')
                d.end_to_end_identifier = d_refs.get('EndToEndId', '')

                d_parties = dtl.get('RltdPties', {})
                d_dbt = d_parties.get('Dbtr', {})
                d.debtor_name = d_dbt.get('Nm', '')
                d_udbt = d_parties.get('UltmtDbtr', {})
                d.ultimate_debtor_name = d_udbt.get('Nm', '')
                d_cdtr = d_parties.get('Cdtr', {})
                d.creditor_name = d_cdtr.get('Nm', '')
                d_cdtr_acct = d_parties.get('CdtrAcct', {})
                d.creditor_account = d_cdtr_acct.get('Id', {}).get('IBAN', '')

                d_rmt = dtl.get('RmtInf', {})
                d.unstructured_remittance_info = d_rmt.get('Ustrd', '')

                d_rltd_dts = dtl.get('RltdDts', {})
                d.paid_date = camt053_get_dt(d_rltd_dts, 'AccptncDtTm', required=False)

                d.full_clean()
                d.save()

                st = StatementRecordRemittanceInfo(detail=d)
                for strd in d_rmt.get('Strd', []):
                    additional_info = strd.get('AddtlRmtInf', '')
                    amount, currency_code = camt053_get_currency(strd.get('RfrdDocAmt', {}), 'RmtdAmt', required=False)
                    reference = strd.get('CdtrRefInf', {}).get('Ref', '')

                    # check if new remittance info record is needed
                    if additional_info and st.additional_info or amount and st.amount or reference and st.reference:
                        st = StatementRecordRemittanceInfo(detail=d)

                    if additional_info:
                        st.additional_info = additional_info
                    if amount:
                        st.amount, st.currency_code = amount, currency_code
                    if reference:
                        st.reference = reference

                    st.full_clean()
                    st.save()

                dtl_ix += 1

        # fill record name from details
        if not rec.name:
            if rec.type.code == e_withdraw.code:
                rec.name = camt053_get_unified_str(rec.detail_set.all(), 'creditor_name')
            elif rec.type.code == e_deposit.code:
                rec.name = camt053_get_unified_str(rec.detail_set.all(), 'debtor_name')
        if not rec.recipient_account_number:
            rec.recipient_account_number = camt053_get_unified_str(rec.detail_set.all(), 'creditor_account')
        if not rec.remittance_info:
            rec.remittance_info = camt053_get_unified_str(StatementRecordRemittanceInfo.objects.all().filter(detail__record=rec), 'reference')
        if not rec.paid_date:
            paid_date = camt053_get_unified_val(rec.detail_set.all(), 'paid_date', default=None)
            if paid_date:
                assert isinstance(paid_date, datetime)
                rec.paid_date = paid_date.date()

        rec.full_clean()
        rec.save()

    return stm




# Generated by Django 2.1.7 on 2019-03-04 19:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0026_auto_20181205_0034'),
    ]

    operations = [
        migrations.AddField(
            model_name='referencepaymentrecord',
            name='manually_settled',
            field=models.BooleanField(blank=True, db_index=True, default=False, verbose_name='manually settled'),
        ),
        migrations.AddField(
            model_name='statementrecord',
            name='manually_settled',
            field=models.BooleanField(blank=True, db_index=True, default=False, verbose_name='manually settled'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-27 16:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0030_auto_20190727_1633'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statementrecord',
            name='record_code',
            field=models.CharField(blank=True, choices=[('700', 'Money Transfer (In/Out)'), ('701', 'Recurring Payment (In/Out)'), ('702', 'Bill Payment (Out)'), ('703', 'Payment Terminal Deposit (In)'), ('704', 'Bank Draft (In/Out)'), ('705', 'Reference Payments (In)'), ('706', 'Payment Service (Out)'), ('710', 'Deposit (In)'), ('720', 'Withdrawal (Out)'), ('721', 'Card Payment (Out)'), ('722', 'Check (Out)'), ('730', 'Bank Fees (Out)'), ('740', 'Interests Charged (Out)'), ('750', 'Interests Credited (In)'), ('760', 'Loan (Out)'), ('761', 'Loan Payment (Out)'), ('770', 'Foreign Transfer (In/Out)'), ('780', 'Zero Balancing (In/Out)'), ('781', 'Sweeping (In/Out)'), ('782', 'Topping (In/Out)')], db_index=True, max_length=4, verbose_name='record type'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-27 13:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0028_auto_20190327_1830'),
    ]

    operations = [
        migrations.CreateModel(
            name='CurrencyExchange',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('record_date', models.DateField(db_index=True, verbose_name='record date')),
                ('source_currency', models.CharField(blank=True, max_length=3, verbose_name='source currency')),
                ('target_currency', models.CharField(blank=True, max_length=3, verbose_name='target currency')),
                ('unit_currency', models.CharField(blank=True, max_length=3, verbose_name='unit currency')),
                ('exchange_rate', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='exchange rate')),
            ],
        ),
        migrations.CreateModel(
            name='StatementRecordDetail',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('batch_identifier', models.CharField(blank=True, db_index=True, default='', max_length=64, verbose_name='batch message id')),
                ('amount', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='amount')),
                ('currency_code', models.CharField(max_length=3, verbose_name='currency code')),
                ('instructed_amount', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='instructed amount')),
                ('archive_identifier', models.CharField(blank=True, max_length=64, verbose_name='archive identifier')),
                ('end_to_end_identifier', models.CharField(blank=True, max_length=64, verbose_name='end-to-end identifier')),
                ('creditor_name', models.CharField(blank=True, max_length=128, verbose_name='creditor name')),
                ('creditor_account', models.CharField(blank=True, max_length=35, verbose_name='creditor account')),
                ('debtor_name', models.CharField(blank=True, max_length=128, verbose_name='debtor name')),
                ('ultimate_debtor_name', models.CharField(blank=True, max_length=128, verbose_name='ultimate debtor name')),
                ('unstructured_remittance_info', models.CharField(blank=True, max_length=128, verbose_name='unstructured remittance info')),
                ('paid_date', models.DateTimeField(blank=True, db_index=True, default=None, null=True, verbose_name='paid date')),
                ('exchange', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='recorddetail_set', to='jbank.CurrencyExchange', verbose_name='currency exchange')),
            ],
        ),
        migrations.AddField(
            model_name='statement',
            name='statement_identifier',
            field=models.CharField(blank=True, db_index=True, default='', max_length=48, verbose_name='statement identifier'),
        ),
        migrations.AddField(
            model_name='statementrecord',
            name='family_code',
            field=models.CharField(blank=True, db_index=True, default='', max_length=4, verbose_name='family code'),
        ),
        migrations.AddField(
            model_name='statementrecord',
            name='record_domain',
            field=models.CharField(blank=True, choices=[('PMNT', 'Money Transfer (In/Out)'), ('LDAS', 'Loan Payment (Out)'), ('CAMT', 'Cash Management'), ('ACMT', 'Account Management'), ('XTND', 'Entended Domain'), ('SECU', 'Securities'), ('FORX', 'Foreign Exchange'), ('XTND', 'Entended Domain'), ('NTAV', 'Not Available')], db_index=True, max_length=4, verbose_name='record domain'),
        ),
        migrations.AddField(
            model_name='statementrecord',
            name='sub_family_code',
            field=models.CharField(blank=True, db_index=True, default='', max_length=4, verbose_name='sub family code'),
        ),
        migrations.AlterField(
            model_name='statement',
            name='bank_specific_info_1',
            field=models.CharField(blank=True, default='', max_length=1024, verbose_name='bank specific info (1)'),
        ),
        migrations.AlterField(
            model_name='statement',
            name='contact_info_1',
            field=models.CharField(blank=True, default='', max_length=64, verbose_name='contact info (1)'),
        ),
        migrations.AlterField(
            model_name='statement',
            name='customer_identifier',
            field=models.CharField(blank=True, default='', max_length=64, verbose_name='customer identifier'),
        ),
        migrations.AlterField(
            model_name='statement',
            name='statement_number',
            field=models.SmallIntegerField(db_index=True, verbose_name='statement number'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='archive_identifier',
            field=models.CharField(blank=True, db_index=True, default='', max_length=64, verbose_name='archive identifier'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='line_number',
            field=models.SmallIntegerField(blank=True, default=None, null=True, verbose_name='line number'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='record_code',
            field=models.CharField(choices=[('700', 'Money Transfer (In/Out)'), ('701', 'Recurring Payment (In/Out)'), ('702', 'Bill Payment (Out)'), ('703', 'Payment Terminal Deposit (In)'), ('704', 'Bank Draft (In/Out)'), ('705', 'Reference Payments (In)'), ('706', 'Payment Service (Out)'), ('710', 'Deposit (In)'), ('720', 'Withdrawal (Out)'), ('721', 'Card Payment (Out)'), ('722', 'Check (Out)'), ('730', 'Bank Fees (Out)'), ('740', 'Interests Charged (Out)'), ('750', 'Interests Credited (In)'), ('760', 'Loan (Out)'), ('761', 'Loan Payment (Out)'), ('770', 'Foreign Transfer (In/Out)'), ('780', 'Zero Balancing (In/Out)'), ('781', 'Sweeping (In/Out)'), ('782', 'Topping (In/Out)')], db_index=True, max_length=4, verbose_name='record type'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='record_description',
            field=models.CharField(blank=True, default='', max_length=128, verbose_name='record description'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='record_number',
            field=models.IntegerField(blank=True, default=None, null=True, verbose_name='record number'),
        ),
        migrations.AlterField(
            model_name='statementrecord',
            name='remittance_info',
            field=models.CharField(blank=True, db_index=True, max_length=35, verbose_name='remittance info'),
        ),
        migrations.AlterField(
            model_name='statementrecordsepainfo',
            name='archive_identifier',
            field=models.CharField(blank=True, max_length=64, verbose_name='archive identifier'),
        ),
        migrations.CreateModel(
            name='StatementRecordRemittanceInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('additional_info', models.CharField(blank=True, db_index=True, max_length=64, verbose_name='additional remittance info')),
                ('amount', models.DecimalField(blank=True, decimal_places=2, default=None, max_digits=10, null=True, verbose_name='amount')),
                ('currency_code', models.CharField(blank=True, max_length=3, verbose_name='currency code')),
                ('reference', models.CharField(blank=True, db_index=True, max_length=35, verbose_name='reference')),
                ('detail', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='remittanceinfo_set', to='jbank.StatementRecordDetail')),
            ],
        ),
        migrations.AddField(
            model_name='statementrecorddetail',
            name='record',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='detail_set', to='jbank.StatementRecord', verbose_name='record'),
        ),
    ]



import os
from copy import copy
from pathlib import Path
from pprint import pprint
from django.core.files import File
from django.core.management import CommandParser
from django.db import transaction
from jbank.camt import camt053_get_iban, camt053_create_statement, camt053_parse_statement_from_file
from jbank.helpers import get_or_create_bank_account
from jbank.files import list_dir_files
from jbank.models import Statement, StatementFile
from jutil.command import SafeCommand


class Command(SafeCommand):
    help = 'Parses XML bank account statement (camt.053.001.02) files'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('path', type=str)
        parser.add_argument('--verbose', action='store_true')
        parser.add_argument('--test', action='store_true')
        parser.add_argument('--delete-old', action='store_true')
        parser.add_argument('--auto-create-accounts', action='store_true')
        parser.add_argument('--suffix', type=str)

    def do(self, *args, **options):
        files = list_dir_files(options['path'], options['suffix'])
        for filename in files:
            plain_filename = os.path.basename(filename)
            if options['delete_old']:
                Statement.objects.filter(name=plain_filename).delete()

            if options['test']:
                statement = camt053_parse_statement_from_file(filename)
                pprint(statement)
                continue

            if not Statement.objects.filter(name=plain_filename).first():
                print('Importing statement file {}'.format(plain_filename))

                statement = camt053_parse_statement_from_file(filename)
                if options['verbose']:
                    pprint(statement)

                with transaction.atomic():
                    if not Statement.objects.filter(name=plain_filename).first():
                        file = StatementFile()
                        file.save()
                        with open(filename, 'rb') as fp:
                            file.file.save(plain_filename, File(fp))
                        for data in [statement]:
                            if options['auto_create_accounts']:
                                account_number = camt053_get_iban(data)
                                if account_number:
                                    get_or_create_bank_account(account_number)

                            camt053_create_statement(data, name=plain_filename, file=file)
            else:
                print('Skipping statement file {}'.format(filename))



# Generated by Django 2.1.7 on 2019-03-27 18:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0027_auto_20190304_1913'),
    ]

    operations = [
        migrations.AlterField(
            model_name='referencepaymentbatch',
            name='institution_identifier',
            field=models.CharField(blank=True, max_length=2, verbose_name='institution identifier'),
        ),
    ]



# Generated by Django 2.2.3 on 2019-08-01 21:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('jbank', '0032_auto_20190727_1655'),
    ]

    operations = [
        migrations.AlterField(
            model_name='currencyexchange',
            name='exchange_rate',
            field=models.DecimalField(blank=True, decimal_places=4, default=None, max_digits=12, null=True, verbose_name='exchange rate'),
        ),
    ]
